#pragma once

#include <any>
#include <string>
#include <vector>
#include <map>
#include <list>
#include <set>
#include <tuple>
#include <memory>
#include <functional>
#include <initializer_list>

#include <a8/args.h>
#include <a8/types.h>
#include <a8/list.h>
#include <a8/xvalue.h>
#include <a8/xobject.h>
#include <a8/timer_attacher.h>

#include <a8/strutils.h>
#include <a8/sysutils.h>

#define A8_SAFE_DELETE(p) { if(p){delete(p);  (p)=nullptr;} }
#define A8_DEFINE_RANGE_BIT(type, begin, end) ((((type)1) << (end + 1)) - 1) & (~((((type)1) << (begin + 0)) - 1))

#define A8_ABORT() do{printf("abort file:%s line:%d func:%s\n", __FILE__, __LINE__, __func__);fflush(stdout);fflush(stderr);abort();}while(0);

#define A8_DECLARE_ENUM(E, ...) \
enum E \
{ \
__VA_ARGS__ \
}; \
template<> constexpr const char* GetEnumString<E>() { return #__VA_ARGS__; };

#define A8_DECLARE_CLASS_ENUM(E, T, ...) \
enum class E : T \
{ \
__VA_ARGS__ \
}; \
template<> constexpr const char* GetEnumString<E>() { return #__VA_ARGS__; };

#define A8_MAKE_ANON_STRUCT_SHARED(...) \
[] ()                                       \
{                                               \
struct Context                                  \
{                                               \
    __VA_ARGS__                                 \
};                                              \
return std::make_shared<Context>();}()

#define A8_MAKE_SMART_ANON_STRUCT_SHARED(...) \
[] ()                                       \
{                                               \
struct Context : public std::enable_shared_from_this<Context>    \
{                                               \
    __VA_ARGS__;                                \
    std::function<void()> _destory_cb;                                   \
    std::weak_ptr<Context> GetWp() { return shared_from_this();};           \
    ~Context() { if (_destory_cb) { _destory_cb(); };};        \
};                                              \
return std::make_shared<Context>();}()

const float A8_PI = 3.1415926f;
