#include <assert.h>

#include <a8/a8.h>

#include <algorithm>
#include <cfloat>
#include <glm/gtx/intersect.hpp>
#include <glm/vec3.hpp>

#include <a8/collision.h>

namespace a8
{

    bool IntersectCylinderCylinder(const glm::vec3& a_pos, float a_rad, float a_height,
                                   const glm::vec3& b_pos, float b_rad, float b_height)
    {
        glm::vec2 v = glm::vec2(b_pos.x, b_pos.z) - glm::vec2(a_pos.x, a_pos.z);
        float n = glm::length(v);

        #ifdef DEBUG1
        a8::XPrintf("a_pos:%f,%f,%f a_rad:%f b_pos:%f,%f,%f b_rad:%f n:%f\n",
                    {
                     a_pos.x,
                     a_pos.y,
                     a_pos.z,
                     a_rad,

                     b_pos.x,
                     b_pos.y,
                     b_pos.z,
                     b_rad,
                     n
                    });
        #endif

        if (n > a_rad + b_rad) {
            return false;
        }
        return (a_pos.y - b_pos.y) < (a_height + b_height) / 2.0f;
    }

    bool CircleContainCircle(glm::vec2 a_pos, float a_rad, glm::vec2 b_pos, float b_rad)
    {
        float distance = glm::length(a_pos - b_pos);
        return distance < a_rad - b_rad;
    }

}
