#include <curl/curl.h>
#include <curl/easy.h>

#include <a8/a8.h>
#include <a8/openssl.h>

namespace a8
{
    namespace http
    {
        static size_t _ReceiveResponse(void *buffer, size_t size, size_t nmemb, void *user_p)
        {
            std::string *respone = (std::string *)user_p;
            respone->append((char*)buffer, size * nmemb);
            return size * nmemb;
        }

        bool Post(const char* url, const std::string& content, std::string& response, a8::XObject* headers,
                  int timeout)
        {
            CURL *curl = curl_easy_init();
            curl_easy_setopt(curl, CURLOPT_TIMEOUT, 10);
            curl_easy_setopt(curl, CURLOPT_URL, url);
            curl_easy_setopt(curl, CURLOPT_HEADER, 0);
            curl_easy_setopt(curl, CURLOPT_HTTPPOST, 1);
            curl_slist* list = nullptr;
            if (headers && headers->GetType() == a8::XOT_ARRAY) {
                for (int i = 0; i < headers->Size(); ++i) {
                    list = curl_slist_append(list, headers->At(i)->AsXValue().GetString().c_str());
                }
                curl_easy_setopt(curl, CURLOPT_HTTPHEADER, list);
            }
            curl_easy_setopt(curl, CURLOPT_POSTFIELDS, content.c_str());
            curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, &_ReceiveResponse);
            curl_easy_setopt(curl, CURLOPT_WRITEDATA, &response);
            bool ok = curl_easy_perform(curl) == CURLE_OK;
            if (list) {
                curl_slist_free_all(list);
            }
            curl_easy_cleanup(curl);
            return ok;
        }

        bool Get(const std::string& url, std::string& response, a8::XObject* headers, int timeout)
        {
            CURL *curl = curl_easy_init();
            curl_easy_setopt(curl, CURLOPT_TIMEOUT, timeout);
            curl_easy_setopt(curl, CURLOPT_URL, url.c_str());
            curl_easy_setopt(curl, CURLOPT_HTTPGET, 1);
            curl_slist* list = nullptr;
            if (headers && headers->GetType() == a8::XOT_ARRAY) {
                for (int i = 0; i < headers->Size(); ++i) {
                    list = curl_slist_append(list, headers->At(i)->AsXValue().GetString().c_str());
                }
                curl_easy_setopt(curl, CURLOPT_HTTPHEADER, list);
            }
            curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, &_ReceiveResponse);
            curl_easy_setopt(curl, CURLOPT_WRITEDATA, &response);
            bool ok = curl_easy_perform(curl) == CURLE_OK;
            if (list) {
                curl_slist_free_all(list);
            }
            curl_easy_cleanup(curl);
            return ok;
        }

    }
}
