#pragma once

#include <arpa/inet.h>
#include <fcntl.h>
#include <sys/epoll.h>
#include <netinet/tcp.h>

namespace a8
{

    enum UDPLISTENER_E
    {
        UE_CREATE_ERR,
        UE_SETSOCKOPT_ERR,
        UE_BIND_ERR,
        UE_LISTEN_ERR,
    };

    struct UdpPacket
    {
        const char* buf = nullptr;
        int buf_len = 0;
        sockaddr_in remote_addr = {};
    };

    class UdpListener
    {
    public:
        std::function<void (int error_id)> on_error;
        std::function<void (UdpPacket*)> on_recv_packet;
        std::string bind_address;
        unsigned short bind_port = 0;

    public:
        UdpListener();
        ~UdpListener();

        void Open();
        void Close();
        bool IsActive();
        virtual void SendUdpPacket(UdpPacket* pkt);

    private:
        std::shared_ptr<struct UdpListenerImpl> impl_;
    };

}
