#include <a8/a8.h>

#include <memory.h>
#include <unistd.h>

#include <a8/udpsession.h>

static const int DEFAULT_MAX_RECV_BUFFERSIZE = 1024 * 64;

namespace a8
{

    UdpSession::UdpSession()
    {
        max_packet_len_ = DEFAULT_MAX_RECV_BUFFERSIZE;
        SetMaxPacketLen(max_packet_len_);
    }

    UdpSession::~UdpSession()
    {
        if (recv_buff_) {
            recv_bufflen_ = 0;
            free(recv_buff_);
            recv_buff_ = nullptr;
        }
    }

    void UdpSession::Update(long long tick)
    {

    }

    void UdpSession::SetMaxPacketLen(int max_packet_len)
    {
        max_packet_len_ = std::max(max_packet_len, DEFAULT_MAX_RECV_BUFFERSIZE);
        if (!recv_buff_) {
            recv_buff_ = (char *)malloc(max_packet_len_ + 1);
        }
        recv_bufflen_ = 0;
    }

    void UdpSession::OnSocketRead(char* buf, unsigned int buflen)
    {
        unsigned int already_read_bytes = 0;
        do {
            if (already_read_bytes < buflen) {
                int read_bytes = std::min(buflen - already_read_bytes,
                                          (unsigned int)max_packet_len_ - recv_bufflen_);
                if (read_bytes > 0) {
                    memmove(&recv_buff_[recv_bufflen_], buf + already_read_bytes, read_bytes);
                    recv_bufflen_ += read_bytes;
                    already_read_bytes += read_bytes;
                }
            }

            int offset = 0;
            int prev_offset = 0;
            do {
                prev_offset = offset;
                DecodePacket(recv_buff_, offset, recv_bufflen_);
            } while (prev_offset < offset && offset < recv_bufflen_);

            if (offset > 0 && offset < recv_bufflen_){
                memmove(recv_buff_, recv_buff_ + offset, recv_bufflen_ - offset);
            }
            recv_bufflen_ -= offset;
            if (recv_bufflen_ >= max_packet_len_) {
                //收到超长包
                #if 0
                Close();
                #endif
                return;
            }
        } while (already_read_bytes < buflen);
    }

    void UdpSession::DecodePacket(char* buf, int& offset, unsigned int buflen)
    {
        DecodeUserPacket(buf, offset, buflen);
    }

}
